/* ZipConstants.java - Some constants used in the zip package
   Copyright (C) 1999, 2000 Free Software Foundation, Inc.

This file is part of GNU Classpath.

GNU Classpath is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.
 
GNU Classpath is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU Classpath; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA.

As a special exception, if you link this library with other files to
produce an executable, this library does not by itself cause the
resulting executable to be covered by the GNU General Public License.
This exception does not however invalidate any other reasons why the
executable file might be covered by the GNU General Public License. */

package java.util.zip;

/**
 * Some constants used in the zip package.
 * <p>
 * Since this package local interface is completely undocumented no effort
 * is made to make it compatible with other implementations.
 * If someone is really interested you can probably come up with the right
 * constants and documentation by studying the Info-ZIP zipfile.c constants.
 */
interface ZipConstants
{
  // Size in bytes of local file header, including signature.
  public static final int LOCAL_FILE_HEADER_SIZE = 30;

  // Size in bytes of the "end of central directory" record, with signature.
  public static final int END_CENTRAL_DIR_SIZE = 22;
}
